@@ -5,6 +5,7 @@
 
     copyright            : (C) 1999 by Martin Lorenz
     email                : lorenz@ch.tum.de
+                           (C) 2002 Matthias Kretz <kretz@kde.org>
  ***************************************************************************/
 
 /***************************************************************************
@@ -19,15 +20,15 @@
 #ifndef _KPOTI_H
 #define _KPOTI_H
 
-#include "qwidget.h"
-#include "qrangecontrol.h"
+#include <qframe.h>
+#include <qrangecontrol.h>
 
 
 class QTimer;
 struct QPotiData;
 
 
-class KPoti : public QWidget, public QRangeControl
+class KPoti : public QFrame, public QRangeControl
 {
     Q_OBJECT
 public:
@@ -36,10 +37,11 @@ public:
     KPoti( int minValue, int maxValue, int step, int value,
 	     QWidget *parent=0, const char *name=0 );
 
+    ~KPoti();
+
     void	setTracking( bool enable );
     bool	tracking() const;
 
-    void 	setPalette( const QPalette & );
     void 	setColor( const QColor & );
 
     virtual void setTickmarks( bool );
@@ -49,10 +51,16 @@ public:
     virtual void setTickInterval( int );
     int 	tickInterval() const { return tickInt; }
 
+    virtual QSizePolicy sizePolicy() const;
+    virtual QSize sizeHint() const;
+    virtual QSize minimumSizeHint() const;
+    QString text() const;
+
 public slots:
     void	setValue( int );
     void	addStep();
     void	subtractStep();
+    void	setText( const QString & );
 
 signals:
     void	valueChanged( int value );
@@ -63,7 +71,7 @@ signals:
 
 protected:
     void	resizeEvent( QResizeEvent * );
-    void	paintEvent( QPaintEvent * );
+    void	drawContents( QPainter * );
 
     void	keyPressEvent( QKeyEvent * );
 
@@ -73,13 +81,16 @@ protected:
     void  enterEvent( QEvent *);
 
     void	focusInEvent( QFocusEvent *e );
+    void	focusOutEvent( QFocusEvent *e );
 
     void	valueChange();
     void	rangeChange();
 
+    virtual void paletteChange( const QPalette & );
+
     virtual void paintPoti( QPainter * );
     void	drawButton( QPainter *);
-    void	drawTicks( QPainter *, int, int, int=1 ) const;
+    void	drawTicks( QPainter *, double, double, int=1 ) const;
 
     virtual void wheelEvent(QWheelEvent *e);
 private slots:
@@ -103,13 +114,14 @@ private:
     int 	clickOffset;
     State	state;
     bool	track;
-    QPoint      center;
-    bool	ticks,label;
-    int		tickInt, space, buttonRadius, fontHeight;
-    QColor      color;
+    bool	ticks, m_bLabel;
+    int		tickInt, space;
+    double buttonRadius;
 private:	// Disabled copy constructor and operator=
     //    KPoti( const KPoti & ) {}
     //KPoti &operator=( const KPoti & ) { return *this; }
+    struct KPotiPrivate;
+    KPotiPrivate * d;
 };
 
 inline bool KPoti::tracking() const
