@@ -51,6 +51,7 @@ public:
 
     QStringList trackProperties();
     QString trackProperty(const QString &property) const;
+    QPixmap trackCover(const QString &size) const;
 
     FileHandle playingFile() const;
     QString playingString() const;
@@ -58,6 +59,8 @@ public:
     void setPlaylistInterface(PlaylistInterface *interface);
     void setStatusLabel(StatusLabel *label);
 
+    QString randomPlayMode() const;
+
     static KSelectAction *playerSelectAction(QObject *parent);
 
 public slots:
@@ -79,6 +82,8 @@ public slots:
     void volumeDown();
     void mute();
 
+    void setRandomPlayMode(const QString &randomMode);
+
 signals:
     void signalPlay();
     void signalPause();
@@ -96,8 +101,6 @@ private slots:
     void slotSetVolume(int volume);
 
 private:
-    static PlayerManager *m_instance;
-
     FileHandle m_file;
     SliderAction *m_sliderAction;
     PlaylistInterface *m_playlistInterface;
@@ -106,6 +109,7 @@ private:
     QTimer *m_timer;
     bool m_noSeek;
     bool m_muted;
+    bool m_setup;
 
     static const int m_pollInterval = 800;
 };
