@@ -1,6 +1,4 @@
 /***************************************************************************
-                          krecexport_wave.h  -  description
-                             -------------------
     copyright            : (C) 2003 by Arnold Krille
     email                : arnold@arnoldarts.de
  ***************************************************************************/
@@ -32,7 +30,7 @@ public:
 	QStringList extensions();
 	QString exportFormat() { return QString( "Wave" ); }
 public slots:
-	bool initialize( QString );
+	bool initialize( const QString & );
 	bool process();
 	bool finalize();
 
