@@ -1,6 +1,4 @@
 /***************************************************************************
-                          main.cpp  -  description
-                             -------------------
     begin                : Mon Jun 17 2002
     copyright            : (C) 2002 by Arnold Krille
     email                : arnold@arnoldarts.de
@@ -42,12 +40,12 @@ static KCmdLineOptions options[] =
 	// INSERT YOUR COMMANDLINE OPTIONS HERE
 };
 
-extern "C" int kdemain( int argc, char* argv[] )
+extern "C" KDE_EXPORT int kdemain( int argc, char* argv[] )
 {
 	//cout<<endl<<"Starting kRec..."<<endl;
 
 	KAboutData aboutData( "krec", I18N_NOOP("KRec"),
-		"0.5", description, KAboutData::License_GPL,
+		"0.5.1", description, KAboutData::License_GPL,
 		"(c) 2002, 2003 Arnold Krille" );
 	aboutData.addAuthor("Arnold Krille", I18N_NOOP( "Creator \nLook at the website www.arnoldarts.de \nfor other good stuff." ), "arnold@arnoldarts.de");
 	aboutData.addCredit( "Matthias Kretz", I18N_NOOP( "Helped where he was asked" ), "kretz@kde.org" );
