@@ -12,7 +12,9 @@
 
 
 #include "yafOutputStream.h"
+#include <iostream>
 
+using namespace std;
 
 #define _AUDIO_DELAY_STAMPS 200
 
