@@ -20,7 +20,7 @@
 #include "pluginInfo.h"
 #include "../util/render/pictureArray.h"
 #include "../util/abstract/abs_thread.h"
-
+#include <kdemacros.h>
 
 #define _OUTPUT_WAIT_METHOD_BLOCK       1
 #define _OUTPUT_WAIT_METHOD_POLL        2
@@ -76,7 +76,7 @@
 
 
 
-class OutputStream {
+class KDE_EXPORT OutputStream {
 
   int audioState;
   int videoState;
