@@ -223,7 +223,8 @@ QPixmap KJFont::drawPixmapFont(const QCString &str, int wide, const QPoint &pos)
 //	kdDebug(66666) << k_funcinfo << "pixmap width=" << region.width() << endl;
 
 	// draw every char and add spacing in between these chars if defined
-	for ( unsigned int charPos=0; charPos < string.length(); charPos++ )
+	unsigned int stringLength(string.length());
+	for ( unsigned int charPos=0; charPos < stringLength; charPos++ )
 	{
 		char c = string[charPos]; // the character to be drawn next
 
