@@ -1,4 +1,4 @@
-/* $Id: oggPlayObject_impl.cpp,v 1.3 2001/12/14 20:54:43 fwolf Exp $ */
+/* $Id: oggPlayObject_impl.cpp 185834 2002-10-27 01:10:28Z pfeiffer $ */
 
 #include <sys/types.h>
 #include <sys/ipc.h>
@@ -62,6 +62,7 @@ oggPlayObject_impl::~oggPlayObject_impl()
 bool oggPlayObject_impl::loadMedia(const std::string &filename)
 {
 	halt(); // stop playing any previous stream
+	currentFile = filename;
 
 	//throw off a process to handle the decoding
 	//fill the buffers first...
@@ -193,7 +194,7 @@ poCapabilities oggPlayObject_impl::capabilities() {
 }
 
 std::string oggPlayObject_impl::mediaName() {
-	return "";
+	return currentFile;
 }
 
 poState oggPlayObject_impl::state() {
